# $RCSfile: activeareaPage.py,v $
# $Revision: 1.18 $
# $Author: langer $
# $Date: 2005/09/07 21:31:55 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

# GUI page for manipulating pixel groups.

from oof2.SWIG.common import switchboard
from oof2.common import activeareamod
from oof2.common import debug
from oof2.common import guitop
from oof2.common import microstructure
from oof2.common import utils
from oof2.common.IO import mainmenu
from oof2.common.IO import questioner
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import historian
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
import gtk

class ActiveAreaPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Active Area", ordering=71.1,
                                 tip="Modify active area.")

        mainbox = gtk.VBox(spacing=2)
        self.gtk.add(mainbox)

        # Microstructure widget, centered at the top of the page.
        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        label = gtk.Label('Microstructure=')
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        self.mswidget = whowidget.WhoWidget(microstructure.microStructures,
                                            scope=self)
        centerbox.pack_start(self.mswidget.gtk[0], expand=0, fill=0)

        mainpane = gtk.HPaned()
        mainbox.pack_start(mainpane, expand=1, fill=1)

        # Active area status in the left half of the main pane.
        vbox = gtk.VBox()
        mainpane.pack1(vbox, resize=1, shrink=0)
        aasframe = gtk.Frame("Active Area Status")
        aasframe.set_shadow_type(gtk.SHADOW_IN)
        vbox.pack_start(aasframe, expand=0, fill=0, padding=2)
        self.aainfo = gtk.Label()
##        self.aainfo.set_alignment(0.0, 0.5)
        aasframe.add(self.aainfo)

        naaframe = gtk.Frame("Named Active Areas")
        naaframe.set_shadow_type(gtk.SHADOW_IN)
        vbox.pack_start(naaframe, expand=1, fill=1)
        naabox = gtk.VBox()
        naaframe.add(naabox)
        self.aalist = chooser.ScrolledChooserListWidget(callback=self.aalistCB,
                                                      dbcallback=self.aalistCB2)
        naabox.pack_start(self.aalist.gtk, expand=1, fill=1, padding=2)
        bbox = gtk.HBox()
        naabox.pack_start(bbox, expand=0, fill=0, padding=2)
        self.storebutton = gtk.Button("Store...")
        bbox.pack_start(self.storebutton, expand=1, fill=0)
        self.storebutton.connect('clicked', self.storeCB)
        guitop.top().tooltip(self.storebutton,
                             "Save the current active area for future use.")
        self.renamebutton = gtk.Button("Rename...")
        bbox.pack_start(self.renamebutton, expand=1, fill=0)
        self.renamebutton.connect('clicked', self.renameCB)
        guitop.top().tooltip(self.renamebutton,
                             "Rename the selected saved active area.")
        self.deletebutton = gtk.Button("Delete")
        bbox.pack_start(self.deletebutton, expand=1, fill=0)
        self.deletebutton.connect('clicked', self.deleteCB)
        guitop.top().tooltip(self.deletebutton,
                             "Delete the selected saved active area.")
        self.restorebutton = gtk.Button("Restore")
        bbox.pack_start(self.restorebutton, expand=1, fill=0)
        self.restorebutton.connect('clicked', self.restoreCB)
        guitop.top().tooltip(self.restorebutton,
                             "Use the selected saved active area.")
        
        # Active area modification methods in the right half of the main pane
        modframe = gtk.Frame("Active Area Modification")
        modframe.set_shadow_type(gtk.SHADOW_IN)
        mainpane.pack2(modframe, resize=0, shrink=0)
        modbox = gtk.VBox()
        modframe.add(modbox)
##        scroll = gtk.ScrolledWindow()
##        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
##        modbox.pack_start(scroll, expand=1, fill=1)
        self.activeareaModFactory = regclassfactory.RegisteredClassFactory(
            activeareamod.ActiveAreaModifier.registry, title="Method:",
            scope=self)
##        scroll.add_with_viewport(self.activeareaModFactory.gtk)
        modbox.pack_start(self.activeareaModFactory.gtk, expand=1, fill=1)
        self.historian = historian.Historian(self.activeareaModFactory.set,
                                             self.sensitizeHistory,
                                             setCBkwargs={'interactive':1})
        self.activeareaModFactory.set_callback(self.historian.stateChangeCB)

        # Prev, OK, and Next buttons
        hbox = gtk.HBox()
        modbox.pack_start(hbox, expand=0, fill=0, padding=2)
        self.prevbutton = gtk.Button()
        lbox = gtk.HBox()
        lbox.pack_start(gtk.Arrow(gtk.ARROW_LEFT, gtk.SHADOW_OUT),
                        expand=0, fill=0)
        lbox.pack_start(gtk.Label(' Prev'), expand=0, fill=0)
        self.prevbutton.add(lbox)
        hbox.pack_start(self.prevbutton, expand=0, fill=0, padding=2)
        self.prevbutton.connect('clicked', self.historian.prevCB)
        guitop.top().tooltip(self.prevbutton,
                     'Recall the previous active area modification operation.')
        self.okbutton = gtk.Button('OK')
        hbox.pack_start(self.okbutton, expand=1, fill=1, padding=2)
        self.okbutton.connect('clicked', self.okbuttonCB)
        guitop.top().tooltip(self.okbutton,
              'Perform the active area modification operation defined above.')
        self.nextbutton = gtk.Button()
        lbox = gtk.HBox()
        lbox.pack_start(gtk.Label('Next '), expand=0, fill=0)
        lbox.pack_start(gtk.Arrow(gtk.ARROW_RIGHT, gtk.SHADOW_OUT),
                        expand=0,fill=0)
        self.nextbutton.add(lbox)
        hbox.pack_start(self.nextbutton, expand=0, fill=0, padding=2)
        self.nextbutton.connect('clicked', self.historian.nextCB)
        guitop.top().tooltip(self.nextbutton,
                        "Recall the next active area modification operation.")

        # Undo, Redo, Override
        hbox = gtk.HBox()
        modbox.pack_start(hbox, expand=0, fill=0, padding=2)
        self.undobutton = gtk.Button('Undo')
        hbox.pack_start(self.undobutton, expand=1, fill=0)
        self.undobutton.connect('clicked', self.undoCB)
        guitop.top().tooltip(self.undobutton, "Undo the previous operation.")

        self.redobutton = gtk.Button('Redo')
        hbox.pack_start(self.redobutton, expand=1, fill=0)
        self.redobutton.connect('clicked', self.redoCB)
        guitop.top().tooltip(self.redobutton, "Redo an undone operation.")

        self.overridebutton = gtk.CheckButton('Override')
        hbox.pack_start(self.overridebutton, expand=1, fill=0)
        self.overridesignal = self.overridebutton.connect('clicked',
                                                          self.overrideCB)
        guitop.top().tooltip(self.overridebutton,
                             "Temporarily activate the entire microstructure.")

        # Switchboard signals
        self.sbcallbacks = [
            switchboard.requestCallback(self.mswidget, self.mswidgetCB),
            switchboard.requestCallback("active area modified",
                                        self.aamodified),
            switchboard.requestCallback("stored active areas changed",
                                        self.storedAAChanged),
            switchboard.requestCallback(('validity', self.activeareaModFactory),
                                        self.validityChangeCB)
        ]

        # Sensitizing
        self.sensitize()
        self.sensitizeHistory()
        
    def getCurrentMSName(self):
        return self.mswidget.get_value()
    def getCurrentMS(self):
        try:
            return microstructure.microStructures[self.getCurrentMSName()]
        except KeyError:
            pass
    def getCurrentActiveArea(self):
        return self.aalist.get_value()

    # Callback functions for active area modification

    def okbuttonCB(self, button):
        modmeth = self.activeareaModFactory.getRegistration()
        if modmeth is not None:
            self.activeareaModFactory.set_defaults()
            menuitem = getattr(mainmenu.OOF.ActiveArea,
                               utils.space2underscore(modmeth.name()))
            menuitem.callWithDefaults(microstructure=self.getCurrentMSName())

    def undoCB(self, button):
        mainmenu.OOF.ActiveArea.Undo.callWithDefaults(
            microstructure=self.getCurrentMSName())

    def redoCB(self, button):
        mainmenu.OOF.ActiveArea.Redo.callWithDefaults(
            microstructure=self.getCurrentMSName())

    def overrideCB(self, button):
        mainmenu.OOF.ActiveArea.Override(microstructure=self.getCurrentMSName(),
                                         override=button.get_active())
        
    def mswidgetCB(self, *args, **kwargs): # switchboard self.mswidget
        self.updateAAInfo()
        self.updateAAList()
        self.sensitize()
        self.setOverrideButton()

    def aalistCB(self, name):           # activearealist callback
        self.sensitize()

    def aalistCB2(self, name):          # activearealist double-click callback
        self.sensitize()
        self.restoreCB(None)
        
    def storeCB(self, button):
        menuitem = mainmenu.OOF.ActiveArea.Store
        if parameterwidgets.getParameters(menuitem.get_arg('name'),
                                          title="Store the active area"):
            menuitem.callWithDefaults(microstructure=self.getCurrentMSName())

    def renameCB(self, button):
        menuitem = mainmenu.OOF.ActiveArea.Rename
        namearg = menuitem.get_arg('newname')
        namearg.value = self.getCurrentActiveArea()
        oldname = self.getCurrentActiveArea()
        if parameterwidgets.getParameters(menuitem.get_arg('newname'),
                                          title="Rename activearea '%s'"
                                               % oldname):
            menuitem.callWithDefaults(microstructure=self.getCurrentMSName(),
                                      oldname=oldname)

    def deleteCB(self, button):
        if questioner.questioner("Really delete %s?" %
                                 self.getCurrentActiveArea(),
                                 "No", default="Yes") == "Yes":
            mainmenu.OOF.ActiveArea.Delete(microstructure=self.getCurrentMSName(),
                                  name=self.getCurrentActiveArea())

    def restoreCB(self, button):
        mainmenu.OOF.ActiveArea.Restore(microstructure=self.getCurrentMSName(),
                               name=self.getCurrentActiveArea())
        

    def aamodified(self, modifier):
        # callback for switchboard signal "active area modified"
        self.updateAAInfo()
        self.updateHistory(modifier)
        self.sensitize()
        self.setOverrideButton()

    def storedAAChanged(self, name):
        # callback for switchboard signal "stored active areas changed"
        self.updateAAList()
        try:
            self.aalist.set_selection(name)
        except KeyError:
            pass
        self.sensitize()

    def updateAAList(self):             # update list of named active areas
        ms = self.getCurrentMS()
        if ms is not None:
            self.aalist.update(ms.getObject().activeAreaNames())
        else:
            self.aalist.update([])

    def updateAAInfo(self):
        ms = self.getCurrentMS()
        if ms is not None:
            activearea = ms.getObject().activearea
            aasize = activearea.size()
            mssize = ms.getObject().sizeInPixels()
            mpxls = mssize[0] * mssize[1]
            apxls = mpxls - aasize
            msg = "%d of %d pixels are active" % (apxls, mpxls)
            self.aainfo.set_text(msg)
        else:
            self.aainfo.set_text("No Microstructure selected")

    def setOverrideButton(self):
        ms = self.getCurrentMS()
        if ms is not None:
            self.overridebutton.handler_block(self.overridesignal)
            self.overridebutton.set_active(
                ms.getObject().activearea.getOverride())
            self.overridebutton.handler_unblock(self.overridesignal)

    def updateHistory(self, activeareaModifier):
        if activeareaModifier is not None:
            self.historian.record(activeareaModifier)
            
    def sensitize(self):
        debug.mainthreadTest()
        ms = self.getCurrentMS()
        if ms is not None:
            activearea = ms.getObject().activearea
            self.undobutton.set_sensitive(activearea.undoable())
            self.redobutton.set_sensitive(activearea.redoable())
            self.overridebutton.set_sensitive(1)
            self.okbutton.set_sensitive(self.activeareaModFactory.isValid())
            self.storebutton.set_sensitive(1)
            aachosen = self.aalist.get_value() is not None
            self.restorebutton.set_sensitive(aachosen)
            self.deletebutton.set_sensitive(aachosen)
            self.renamebutton.set_sensitive(aachosen)
        else:
            self.undobutton.set_sensitive(0)
            self.redobutton.set_sensitive(0)
            self.overridebutton.set_sensitive(0)
            self.okbutton.set_sensitive(0)
            self.storebutton.set_sensitive(0)
            self.restorebutton.set_sensitive(0)
            self.deletebutton.set_sensitive(0)
            self.renamebutton.set_sensitive(0)

    def sensitizeHistory(self):
        debug.mainthreadTest()
        self.nextbutton.set_sensitive(self.historian.nextSensitive())
        self.prevbutton.set_sensitive(self.historian.prevSensitive())

    def validityChangeCB(self, validity):
        self.sensitize()

####################################
        
aap = ActiveAreaPage()
