# -*- python -*-
# $RCSfile: pinnodesGUI.py,v $
# $Revision: 1.23 $
# $Author: langer $
# $Date: 2005/08/08 21:39:34 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import utils
from oof2.common import primitives
from oof2.common.IO import mainmenu
from oof2.common.IO import reporter
from oof2.common.IO.GUI import mousehandler
from oof2.common.IO.GUI import toolboxGUI
from oof2.engine.IO import pinnodes
import gtk

class PinnedNodesToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, pinnodestoolbox):
        debug.mainthreadTest()

        toolboxGUI.GfxToolbox.__init__(self, "Pin Nodes", pinnodestoolbox)
        mainbox = gtk.VBox()
        self.gtk.add(mainbox)

        infoframe = gtk.Frame()
        mainbox.pack_start(infoframe, expand=0, fill=0)
        info = gtk.Label("""Click a node to pin it,
Shift-click to unpin it,
And Ctrl-click to toggle.""")
        infoframe.add(info)
            

        self.table = gtk.Table(columns=3, rows=5)
        mainbox.pack_start(self.table, expand=0, fill=0)

        label = gtk.Label('Mouse')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 0,2, xpadding=2, xoptions=0)

        label = gtk.Label('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 1,2, 0,1, xpadding=5, xoptions=gtk.FILL)
        self.xtext = gtk.Entry()
        self.xtext.set_size_request(12*guitop.top().digitsize, -1)
        self.xtext.set_editable(0)
        self.table.attach(self.xtext, 2,3, 0,1,
                          xpadding=2, xoptions=gtk.EXPAND|gtk.FILL)
        guitop.top().tooltips.set_tip(self.xtext, "x position of the mouse")

        label = gtk.Label('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 1,2, 1,2, xpadding=2, xoptions=gtk.FILL)
        self.ytext = gtk.Entry()
        self.ytext.set_size_request(12*guitop.top().digitsize, -1)
        self.ytext.set_editable(0)
        self.table.attach(self.ytext, 2,3, 1,2,
                          xpadding=2, xoptions=gtk.EXPAND|gtk.FILL)
        guitop.top().tooltips.set_tip(self.ytext, "y position of the mouse")

        self.table.set_row_spacing(1, 5)

        label = gtk.Label("Node")
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 0,1, 2,4, xpadding=2, xoptions=0)

        label = gtk.Label('x=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 1,2, 2,3, xpadding=2, xoptions=gtk.FILL)
        self.nodextext = gtk.Entry()
        self.nodextext.set_size_request(12*guitop.top().digitsize, -1)
        self.nodextext.set_editable(0)
        self.table.attach(self.nodextext, 2,3, 2,3,
                          xpadding=2, xoptions=gtk.EXPAND|gtk.FILL)

        label = gtk.Label('y=')
        label.set_alignment(1.0, 0.5)
        self.table.attach(label, 1,2, 3,4, xpadding=2, xoptions=gtk.FILL)
        self.nodeytext = gtk.Entry()
        self.nodeytext.set_size_request(12*guitop.top().digitsize, -1)        
        self.nodeytext.set_editable(0)
        self.table.attach(self.nodeytext, 2,3, 3,4,
                          xpadding=2, xoptions=gtk.EXPAND|gtk.FILL)

        self.pintext = gtk.Label()
        self.pintext.set_alignment(0.0, 0.5)
        self.table.attach(self.pintext, 2,3, 4,5,
                          xpadding=5, xoptions=gtk.EXPAND|gtk.FILL)

        modbox = gtk.VBox()
        mainbox.pack_end(modbox, expand=0, fill=0)

        bbox1 = gtk.HBox(homogeneous=True, spacing=2)
        modbox.pack_end(bbox1, expand=0, fill=0, padding=2)

        self.undobutton = gtk.Button("Undo")
        self.undobutton.connect("clicked", self.undoCB)
        guitop.top().tooltip(self.undobutton, "Undo the latest action.")
        bbox1.pack_start(self.undobutton, expand=1, fill=1)

        self.redobutton = gtk.Button("Redo")
        self.redobutton.connect("clicked", self.redoCB)
        guitop.top().tooltip(self.redobutton, "Redo the latest undone action.")
        bbox1.pack_start(self.redobutton, expand=1, fill=1)

        bbox2 = gtk.HBox(homogeneous=True, spacing=2)
        modbox.pack_end(bbox2, expand=0, fill=0, padding=2)

        self.unpinallbutton = gtk.Button("UnPinAll")
        self.unpinallbutton.connect("clicked", self.unpinallCB)
        guitop.top().tooltip(self.unpinallbutton, "Unpin all the pinned nodes.")
        bbox2.pack_start(self.unpinallbutton, expand=1, fill=1)

        self.invertbutton = gtk.Button("Invert")
        self.invertbutton.connect("clicked", self.invertCB)
        guitop.top().tooltip(self.invertbutton, "Invert - pin the unpinned and unpin the pinned.")
        bbox2.pack_start(self.invertbutton, expand=1, fill=1)

        self.status = gtk.Label()
        self.status.set_alignment(0.0, 0.5)
        mainbox.pack_end(self.status, expand=0, fill=0, padding=5)

    def update(self, *args):
        debug.mainthreadTest()
        context = self.toolbox.getSkeletonContext()
        if context:
            n = len(context.pinnednodes.retrieve())
        else:
            n = 0
        self.status.set_text("%d node%s pinned." % (n, 's'*(n!=1)))

        self.unpinallbutton.set_sensitive(not not context and n > 0)
        self.undobutton.set_sensitive(not not context and
                                      context.pinnednodes.undoable())
        self.redobutton.set_sensitive(not not context and
                                      context.pinnednodes.redoable())
        self.invertbutton.set_sensitive(not not context)
        
    def skelChanged(self, skelcontext):
        if skelcontext is self.toolbox.getSkeletonContext():
            self.update()
            
    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)
        self.sbcallbacks = [
            switchboard.requestCallback(('who changed', 'Skeleton'),
                                        self.skelChanged),
            switchboard.requestCallback((self.gfxwindow(), "new nodes pinned"),
                                        self.update)
            
            ]
        self.update()

    def deactivate(self):
        toolboxGUI.GfxToolbox.deactivate(self)
        self.gfxwindow().removeMouseHandler()
        map(switchboard.removeCallback, self.sbcallbacks)

    def showPosition(self, x, y):
        debug.mainthreadTest()
        self.xtext.set_text("%-11.4g" % x)
        self.ytext.set_text("%-11.4g" % y)

    def acceptEvent(self, eventtype):
        return eventtype in ('move', 'up')

    def up(self, x, y, shift, ctrl):
        self.showPosition(x, y)
        # Perhaps shift and ctrl should be handled by the menu
        # commands, as they are in pixel selection.
        if shift:
            self.toolbox.menu.UnPin(point=primitives.Point(x, y))
        elif ctrl:
            self.toolbox.menu.TogglePin(point=primitives.Point(x, y))
        else:
            self.toolbox.menu.Pin(point=primitives.Point(x, y))

    def move(self, x, y, shift, ctrl):
        debug.mainthreadTest()
        self.xtext.set_text("%-11.4g" % x)
        self.ytext.set_text("%-11.4g" % y)
        skel = self.toolbox.getSkeleton()
        if skel:
            node = skel.nearestNode(primitives.Point(x,y))
            if node:
                pos = node.position()
                self.nodextext.set_text("%-11.4g" % pos.x)
                self.nodeytext.set_text("%-11.4g" % pos.y)
                if node.pinned():
                    self.pintext.set_text('pinned')
                else:
                    self.pintext.set_text('not pinned')
                return
        self.nodextext.set_text('')
        self.nodeytext.set_text('')
        self.pintext.set_text('')

    def invertCB(self, button):
        self.toolbox.menu.Invert()
    
    def undoCB(self, button):
        self.toolbox.menu.Undo()

    def unpinallCB(self, button):
        self.toolbox.menu.UnPinAll()

    def redoCB(self, button):
        self.toolbox.menu.Redo()

def _makeGUI(self):
    return PinnedNodesToolboxGUI(self)

pinnodes.PinnedNodesToolbox.makeGUI = _makeGUI
