# -*- python -*-
# $RCSfile: tensorwidgets.py,v $
# $Revision: 1.20 $
# $Author: langer $
# $Date: 2005/08/08 21:39:35 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# Contains size-specific descendants of SymmetricMatrixInput,
# which have as values objects of the SymmMatrix type
# from SWIG.engine.SymmMatrix.

from oof2.SWIG.engine import fieldindex
from oof2.SWIG.engine import symmmatrix
from oof2.common import debug
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI.matrixparamwidgets import SymmetricMatrixInput

## Used for triclinic materials with symmetric rank two tensors
class SymmMatrix3Widget(SymmetricMatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        SymmetricMatrixInput.__init__(self,"",3,3,value=None,scope=scope)
        for i in symmmatrix.voigtIndices:
            self.floats[i].gtk.connect("activate", self.new_value, None)
            self.floats[i].gtk.connect("focus_out_event", self.new_value)
        self.set_values(param.value)
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            for i in symmmatrix.voigtIndices:
                self.floats[i].set_value(self.value.get(*i))

        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = symmmatrix.SymmMatrix3(*[self.floats[i].get_value()
                                          for i in symmmatrix.voigtIndices])
        self.set_values(result)

def _SymmMatrix3Parameter_makeWidget(self, scope=None):
    return SymmMatrix3Widget(self, scope)

symmmatrix.SymmMatrix3Parameter.makeWidget = _SymmMatrix3Parameter_makeWidget

class TriclinicSymmWidget(SymmetricMatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        SymmetricMatrixInput.__init__(self,"",3,3,value=None,scope=scope)
        for i in symmmatrix.voigtIndices:
            self.floats[i].gtk.connect("activate", self.new_value, None)
            self.floats[i].gtk.connect("focus_out_event", self.new_value)
        self.set_values(param.value)
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            for i in symmmatrix.voigtIndices:
                self.floats[i].set_value(self.value.get(*i))

        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = symmmatrix.TriclinicRank2Tensor(*[self.floats[i].get_value()
                                          for i in symmmatrix.voigtIndices])
        self.set_values(result)

def _TriclinicSymmParameter_makeWidget(self, scope=None):
    return TriclinicSymmWidget(self, scope)

symmmatrix.TriclinicRank2TensorParameter.makeWidget = _TriclinicSymmParameter_makeWidget
                                        


## for symmetric monoclinic rank two tensors
class MonoclinicSymmWidget(SymmetricMatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        SymmetricMatrixInput.__init__(self,"",3,3,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k==(0,1) or k==(1,2):
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                pass
        for i in symmmatrix.voigtIndices:
            self.floats[i].gtk.connect("activate", self.new_value, None)
            self.floats[i].gtk.connect("focus_out_event", self.new_value)
        self.set_values(param.value)
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,0)].set_value(self.value.get(0,0))
            self.floats[(1,1)].set_value(self.value.get(1,1))
            self.floats[(2,2)].set_value(self.value.get(2,2))
            self.floats[(0,2)].set_value(self.value.get(0,2))
        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = symmmatrix.MonoclinicRank2Tensor(self.floats[(0,0)].get_value(), self.floats[(1,1)].get_value(), self.floats[(2,2)].get_value(), self.floats[(0,2)].get_value())
        self.set_values(result)

def _MonoclinicSymmParameter_makeWidget(self, scope=None):
    return MonoclinicSymmWidget(self, scope)

symmmatrix.MonoclinicRank2TensorParameter.makeWidget = _MonoclinicSymmParameter_makeWidget


## for symmetric orthorhombic rank two tensors
class OrthorhombicSymmWidget(SymmetricMatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        SymmetricMatrixInput.__init__(self,"",3,3,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k==(0,1) or k==(1,2) or k==(0,2) :
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                pass
        for i in symmmatrix.voigtIndices:
            self.floats[i].gtk.connect("activate", self.new_value, None)
            self.floats[i].gtk.connect("focus_out_event", self.new_value)
        self.set_values(param.value)
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,0)].set_value(self.value.get(0,0))
            self.floats[(1,1)].set_value(self.value.get(1,1))
            self.floats[(2,2)].set_value(self.value.get(2,2))

        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = symmmatrix.OrthorhombicRank2Tensor(self.floats[(0,0)].get_value(), self.floats[(1,1)].get_value(), self.floats[(2,2)].get_value())
        self.set_values(result)

def _OrthorhombicSymmParameter_makeWidget(self, scope=None):
    return OrthorhombicSymmWidget(self, scope)

symmmatrix.OrthorhombicRank2TensorParameter.makeWidget = _OrthorhombicSymmParameter_makeWidget


## For Tetragonal, Trigonal, Hexagonal symmetric rank two tensors
## for symmetric orthorhombic rank two tensors
class IsotropicPlaneSymmWidget(SymmetricMatrixInput):
    ## base class for Tetragonal, Trigonal, Hexagonal SymmWidgets.
    ## the function new_value is overwritten by subclasses
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        SymmetricMatrixInput.__init__(self,"",3,3,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k==(0,1) or k==(1,2) or k==(0,2) or k==(1,1):
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                pass
        for i in symmmatrix.voigtIndices:
            self.floats[i].gtk.connect("activate", self.new_value, None)
            self.floats[i].gtk.connect("focus_out_event", self.new_value)
        ##
        self.floats[(0,0)].gtk.connect("activate",self.new_kappa11, None)
        self.floats[(0,0)].gtk.connect("focus_out_event",self.new_kappa11)
        
        self.set_values(param.value)
    def new_kappa11(self, gtk, event):
        kappa11 =  self.floats[(0,0)].get_value()
        self.floats[(0,0)].set_value(kappa11)
        self.floats[(1,1)].set_value(kappa11)
        self.new_value(gtk, event)
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,0)].set_value(self.value.get(0,0))
            self.floats[(1,1)].set_value(self.value.get(0,0))
            self.floats[(2,2)].set_value(self.value.get(2,2))

        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = symmmatrix.IsotropicPlaneRank2Tensor(xx= self.floats[(0,0)].get_value(), zz= self.floats[(2,2)].get_value())
        self.set_values(result)

def _IsotropicPlaneSymmParameter_makeWidget(self, scope=None):
    return IsotropicPlaneSymmWidget(self, scope)

symmmatrix.IsotropicPlaneParameter.makeWidget = _IsotropicPlaneSymmParameter_makeWidget

class TetragonalSymmWidget(IsotropicPlaneSymmWidget):
    def new_value(self, gtk, event):
        result = symmmatrix.TetragonalRank2Tensor(xx= self.floats[(0,0)].get_value(), zz= self.floats[(2,2)].get_value())
        self.set_values(result)
def _TetragonalSymmParameter_makeWidget(self, scope=None):
    return TetragonalSymmWidget(self, scope)

symmmatrix.TetragonalRank2TensorParameter.makeWidget = _TetragonalSymmParameter_makeWidget

class TrigonalSymmWidget(IsotropicPlaneSymmWidget):
    def new_value(self, gtk, event):
        result = symmmatrix.TrigonalRank2Tensor(xx= self.floats[(0,0)].get_value(), zz= self.floats[(2,2)].get_value())
        self.set_values(result)
def _TrigonalSymmParameter_makeWidget(self, scope=None):
    return TrigonalSymmWidget(self, scope)

symmmatrix.TrigonalRank2TensorParameter.makeWidget = _TrigonalSymmParameter_makeWidget

class HexagonalSymmWidget(IsotropicPlaneSymmWidget):
    def new_value(self, gtk, event):
        result = symmmatrix.HexagonalRank2Tensor(xx= self.floats[(0,0)].get_value(), zz= self.floats[(2,2)].get_value())
        self.set_values(result)
def _HexagonalSymmParameter_makeWidget(self, scope=None):
    return HexagonalSymmWidget(self, scope)

symmmatrix.HexagonalRank2TensorParameter.makeWidget = _HexagonalSymmParameter_makeWidget

## For CUBIC symmetric rank two tensors
class CubicSymmWidget(SymmetricMatrixInput):
    def __init__(self, param, scope=None):
        debug.mainthreadTest()
        SymmetricMatrixInput.__init__(self,"",3,3,value=None,scope=scope)
        for (k,f) in self.floats.items():
            if k==(0,1) or k==(1,2) or k==(0,2) or k==(1,1) or k==(2,2):
                f.gtk.set_editable(0)
                f.gtk.set_sensitive(0)
            else:
                pass
        for i in symmmatrix.voigtIndices:
            self.floats[i].gtk.connect("activate", self.new_value, None)
            self.floats[i].gtk.connect("focus_out_event", self.new_value)
        ##
        self.floats[(0,0)].gtk.connect("activate",self.new_kappaxx, None)
        self.floats[(0,0)].gtk.connect("focus_out_event",self.new_kappaxx)
        
        self.set_values(param.value)
    def new_kappaxx(self, gtk, event):
        kappaxx =  self.floats[(0,0)].get_value()
        self.floats[(0,0)].set_value(kappaxx)
        self.floats[(1,1)].set_value(kappaxx)
        self.floats[(2,2)].set_value(kappaxx)
        self.new_value(gtk, event)
        
    def set_values(self, value):
        debug.mainthreadTest()
        self.value = value
        if value is not None:
            self.floats[(0,0)].set_value(self.value.get(0,0))
            self.floats[(1,1)].set_value(self.value.get(0,0))
            self.floats[(2,2)].set_value(self.value.get(0,0))
                                         
        self.gtk.show_all()
    def get_value(self):
        return self.value
    def new_value(self, gtk, event):
        result = symmmatrix.CubicRank2Tensor(self.floats[(0,0)].get_value())
                                          
        self.set_values(result)

def _CubicSymmParameter_makeWidget(self, scope=None):
    return CubicSymmWidget(self, scope)

symmmatrix.CubicRank2TensorParameter.makeWidget = _CubicSymmParameter_makeWidget
