# -*- python -*-
# $RCSfile: solvermenu.py,v $
# $Revision: 1.96 $
# $Author: langer $
# $Date: 2005/09/02 21:27:44 $


# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import devel
from oof2.SWIG.common import switchboard
from oof2.SWIG.common.IO import progressbar
from oof2.SWIG.engine import solver
from oof2.SWIG.engine import solverdriver
from oof2.SWIG.engine import preconditioner
from oof2.common import debug
from oof2.common import utils
from oof2.common import thread_enable
from oof2.common.IO import automatic
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO import parameter
from oof2.common.IO import whoville
from oof2.common.IO import xmlmenudump
from oof2.engine import fieldinit
from oof2.engine import skeletoncontext
from oof2.engine import timedrivers
from types import *
import oof2.engine.mesh



try:
    from oof2.SWIG.engine.PETSc import petsc_solver
except ImportError:
    pass


solvermenu = mainmenu.OOF.addItem(oofmenu.OOFMenuItem(
    'Solver',
    help="Commands for solving Equations on Meshes.",
    cli_only=1))

if devel.devel():
    def _solve(menuitem, mesh, matrixdriver, timedriver):
        solverProgressBar = menuitem.getProgressBar("continuous") ## actually ran by the worker
        meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
        meshcontext.reserve()
        try:
            timedriver.apply(matrixdriver, meshcontext)
            ## solved flag is set even if solution did NOT
            ## converge in order to show how far the solution
            ## went.
            meshcontext.solved()

        finally:
            meshcontext.cancel_reservation()

    solvermenu.addItem(oofmenu.OOFMenuItem(
        'Solve',
        callback=_solve,
        threadable= oofmenu.THREADABLE,
        params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                      tip=parameter.emptyTipString),
                parameter.RegisteredParameter('matrixdriver', solverdriver.Driver,
                                              tip=parameter.emptyTipString),
                parameter.RegisteredParameter('timedriver', timedrivers.TimeDriver)
                ],
        help="Solve a problem",
        discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/menu/solve.xml')
        ))
else:                                   # not devel
    def _solve(menuitem, mesh, solver):
        solverProgressBar = menuitem.getProgressBar("continuous") ## actually ran by the worker
        meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
        meshcontext.reserve()
        timedriver = timedrivers.Null()
        try:
            timedriver.apply(solver, meshcontext)
            ## solved flag is set even if solution did NOT
            ## converge in order to show how far the solution
            ## went.
            meshcontext.solved()

        finally:
            meshcontext.cancel_reservation()
    solvermenu.addItem(oofmenu.OOFMenuItem(
        'Solve',
        callback=_solve,
        threadable= oofmenu.THREADABLE,
        params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                      tip=parameter.emptyTipString),
                parameter.RegisteredParameter('solver', solverdriver.Driver,
                                              tip=parameter.emptyTipString)
                ],
        help="Solve a problem",
        discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/menu/solve.xml')
        ))
    



#######################################

def _reset_mesh(menuitem, mesh):
    meshcontext = oof2.engine.mesh.meshes[mesh]
    meshcontext.reserve()
    try:
        femesh = meshcontext.getObject()
        ## first set time indices back to zero
        femesh.set_time(0)
        ## next set active fields back to zero
        resetObj = fieldinit.ResetFieldInit()
        for field in femesh.all_fields():
            if femesh.is_active_field(field):
                resetObj.apply(femesh, field)
        switchboard.notify("field initialized")
        ## then apply any initializers that had been
        ## applied
        initializers_dict = meshcontext.initializers
        for field in initializers_dict.keys():
            initializer = initializers_dict[field]
            try:
                initializer.apply(femesh, field)
            except:
                reporter.warning("could not re-apply initializer: ",  initializer)
        ## finally notify that solution is not current and
        ## redraw
        meshcontext.solution_is_current = 0
        meshcontext.changed()
        switchboard.notify("redraw")
    finally:
        meshcontext.cancel_reservation()
solvermenu.addItem(oofmenu.OOFMenuItem(
    'Reset',
    callback=_reset_mesh,
    threadable= oofmenu.THREADABLE,
    params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                  tip=parameter.emptyTipString),
            
            ],
    help="Reset the fields on the mesh to their initial values"
    ))

#######################################
#######################################

if devel.devel():
    def _select_elements(femesh, skeletonelements):
        for sel in skeletonelements:
            realel = femesh.getElement(sel.meshindex)
            realel.setsubmesh()
            
    def _unselect_elements(femesh, skeletonelements):
        for sel in skeletonelements:
            realel = femesh.getElement(sel.meshindex)
            realel.unsetsubmesh()
            
    def _problem_solve(menuitem, mesh, matrixdriver, timedriver):
        solverProgressBar = menuitem.getProgressBar("continuous") ## actually ran by the worker
        meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
        meshcontext.reserve()
        try:
            timedriver.apply(matrixdriver, meshcontext)
            ## solved flag is set even if solution did NOT
            ## converge in order to show how far the solution
            ## went.
            meshcontext.solved()
        
        finally:
            meshcontext.cancel_reservation()
## SolveSubProblem does not increment the time index.
## Instead, it applies the solver, and expects the time index
## to be increased separately.
    solvermenu.addItem(oofmenu.OOFMenuItem(
        'SolveSubProblemList',
        callback=_problem_solve,
        threadable= oofmenu.THREADABLE,
        params=[whoville.WhoParameter('mesh', oof2.engine.mesh.meshes,
                                      tip=parameter.emptyTipString),
                parameter.RegisteredParameter('matrixdriver', solverdriver.Driver,
                                              tip=parameter.emptyTipString),
                parameter.RegisteredParameter('timedriver', timedrivers.TimeDriver)
                ],
        help="Solve a set of problems"
        ))
    
