/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*             ********   ***                                 SparseLib++    */
/*          *******  **  ***       ***      ***               v. 1.5c        */
/*           *****      ***     ******** ********                            */
/*            *****    ***     ******** ********              R. Pozo        */
/*       **  *******  ***   **   ***      ***                 K. Remington   */
/*        ********   ********                                 A. Lumsdaine   */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*                                                                           */
/*                                                                           */
/*                     SparseLib++ : Sparse Matrix Library                   */
/*                                                                           */
/*               National Institute of Standards and Technology              */
/*                        University of Notre Dame                           */
/*              Authors: R. Pozo, K. Remington, A. Lumsdaine                 */
/*                                                                           */
/*                                 NOTICE                                    */
/*                                                                           */
/* Permission to use, copy, modify, and distribute this software and         */
/* its documentation for any purpose and without fee is hereby granted       */
/* provided that the above notice appear in all copies and supporting        */
/* documentation.                                                            */
/*                                                                           */
/* Neither the Institutions (National Institute of Standards and Technology, */
/* University of Notre Dame) nor the Authors make any representations about  */
/* the suitability of this software for any purpose.  This software is       */
/* provided ``as is'' without expressed or implied warranty.                 */
/*                                                                           */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*           Compressed row sparse matrix (0-based)                      */
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <oofconfig.h>

#include <iostream>
#include <stdlib.h>
//#include "compcol_double.h" // OOF2
#include "comprow_double.h"
//#include "coord_double.h"  // OOF2

#include "spblas.h"
#include "common/ooferror.h"
using namespace std;		// OOF2

/*****************************/
/*  Constructor(s)           */
/*****************************/

CompRow_Mat_double::CompRow_Mat_double(void)
        : val_(0), rowptr_(0), colind_(0), base_(0), nz_(0)
{
        dim_[0] = 0;
        dim_[1] = 0;
}

/*****************************/
/*  Copy constructor         */
/*****************************/

CompRow_Mat_double::CompRow_Mat_double(const CompRow_Mat_double &S) :  
        val_(S.val_), rowptr_(S.rowptr_), colind_(S.colind_), base_(S.base_), 
        nz_(S.nz_)
{
        dim_[0] = S.dim_[0];
        dim_[1] = S.dim_[1];
}

/***********************************/
/*  Construct from storage vectors */
/***********************************/

CompRow_Mat_double::CompRow_Mat_double(int M, int N, int nz, double *val, 
                                     int *r, int *c, int base) :
        val_(val, nz), rowptr_(r, M+1), colind_(c, nz), base_(base), nz_(nz) 
{
        dim_[0] = M;
        dim_[1] = N;
}

CompRow_Mat_double::CompRow_Mat_double(int M, int N, int nz, 
                   const VECTOR_double &val, const VECTOR_int &r, 
                   const VECTOR_int &c, int base) :
        val_(val), rowptr_(r), colind_(c), base_(base), nz_(nz)
{
        dim_[0] = M;
        dim_[1] = N;
} 


// Commented out for OOF2

// /**********************************************************************/
// /*  Construct a CompRow_Mat_double from a CompCol_Mat_double            */
// /*  (i.e. convert compressed column storage to compressed row storage)*/
// /**********************************************************************/

// CompRow_Mat_double::CompRow_Mat_double(const CompCol_Mat_double &C) : 
//          val_(C.NumNonzeros()), rowptr_(C.dim(0) +1), 
//         colind_(C.NumNonzeros()), base_(C.base()), nz_(C.NumNonzeros())
// {

//         dim_[0] = C.dim(0);
//         dim_[1] = C.dim(1);
        
//         int i,j;

//         VECTOR_int tally(C.dim(0)+1, 0);
// //      First pass through nonzeros.  Tally entries in each row.
// //      And calculate rowptr array.
//         for (i=0;i<nz_;i++) tally(C.row_ind(i))++;
//         rowptr_(0) = 0;
//         for (j=0;j<dim_[0];j++) rowptr_(j+1) = rowptr_(j)+tally(j);
// //      Make copy of rowptr for use in second pass.
//         tally = rowptr_;
 
// //      Second pass through nonzeros.   Fill in index and value entries.
//         int count = 0;
//         for (i=1;i<=dim_[1];i++)
//         {
//            for (j=count;j<C.col_ptr(i);j++)
//            {
//               val_(tally(C.row_ind(j))) = C.val(j);
//               colind_(tally(C.row_ind(j))) = i-1;
//               tally(C.row_ind(count))++;
//               count++;
//            }
//         }
// }

// /**********************************************************************/
// /*  Construct a CompRow_Mat_double from a Coord_Mat_double              */
// /*  (i.e. convert coordinate storage to compressed row storage)       */
// /**********************************************************************/

// CompRow_Mat_double::CompRow_Mat_double(const Coord_Mat_double &CO) : 
//         val_(CO.NumNonzeros()), rowptr_(CO.dim(0)+1), 
//         colind_(CO.NumNonzeros()), base_(CO.base()), nz_(CO.NumNonzeros())
// {

//         dim_[0] = CO.dim(0);
//         dim_[1] = CO.dim(1);

//         int i;
//         VECTOR_int tally(CO.dim(0)+1, 0);
// //      First pass through nonzeros.  Tally entries in each row.
// //      And calculate rowptr array.
//         for (i=0;i<nz_;i++) tally(CO.row_ind(i))++;
//         rowptr_(0) = 0;
//         for (i=0;i<dim_[0];i++) rowptr_(i+1) = rowptr_(i)+tally(i);
// //      Make copy of rowptr for use in second pass.
//         tally = rowptr_;
// //      Second pass through nonzeros.   Fill in index and value entries.
//         for (i=0;i<nz_;i++)
//         {
//            val_(tally(CO.row_ind(i))) = CO.val(i);
//            colind_(tally(CO.row_ind(i))) = CO.col_ind(i);
//            tally(CO.row_ind(i))++;
//         }
// }
              

/***************************/
/* Assignment operator...  */
/***************************/

CompRow_Mat_double& CompRow_Mat_double::operator=(const CompRow_Mat_double &R)
{
        dim_[0] = R.dim_[0];
        dim_[1] = R.dim_[1];
        base_   = R.base_;
        nz_     = R.nz_;
        val_    = R.val_;
        rowptr_ = R.rowptr_;
        colind_ = R.colind_;
        return *this;
}

/***************************/
/* newsize()               */
/***************************/

CompRow_Mat_double& CompRow_Mat_double::newsize(int M, int N, int nz)
{
        dim_[0] = M;
        dim_[1] = N;
        nz_     = nz;
        val_.newsize(nz);
        rowptr_.newsize(M+1);
        colind_.newsize(nz);
        return *this;
}

/*********************/
/*   Array access    */
/*********************/


double CompRow_Mat_double::operator()(int i, int j)  const
{
        for (int t=rowptr_(i); t<rowptr_(i+1); t++)
           if (colind_(t) == j) return val_(t);
        if (i < dim_[0] && j < dim_[1]) return 0.0;
        else
        {
            cerr << "Array accessing exception -- out of bounds." << endl;
            return (0);
        }
}


double& CompRow_Mat_double::set(int i, int j)
{
        for (int t=rowptr_(i); t<rowptr_(i+1); t++)
           if (colind_(t) == j) return val_(t);
        cerr << "Array element (" << i << "," << j << 
                                ") not in sparse structure -- cannot assign."
             << endl;          
        exit(1);
    return val_(0);   // // return to suppress compiler warning message
}

/*************/
/*   I/O     */
/*************/

ostream& operator << (ostream & os, const CompRow_Mat_double & mat)
{
  // Modifications for OOF2:

  //   The original version printed the (M-1,N-1) component even if it
  //   was zero.  To restore this behavior, uncomment the lines
  //   referring to "flag".

  //   The original printed 1-offset fortran indices.  This version
  //   prints 0-offset C indices.

        int M = mat.dim(0);
        int N = mat.dim(1);
//         int flag = 0;
        ios::fmtflags olda = os.setf(ios::right,ios::adjustfield);
        ios::fmtflags oldf = os.setf(ios::scientific,ios::floatfield);
        int oldp = os.precision(12);


//      Loop through rows...
        for (int i = 0; i < M ; i++)
           for (int j=mat.row_ptr(i);j<mat.row_ptr(i+1);j++)
           {   
//               if ( rowp1 == M && colp1 == N ) flag = 1;
              os.width(14);
              os <<  i ; os << "    " ;
              os.width(14);
              os <<  mat.col_ind(j) ; os << "    " ;
              os.width(20);
              os <<  mat.val(j) << "\n";
           }

//         if (flag == 0)
//         {
//            os.width(14);
//            os <<  M ; os << "    " ;
//            os.width(14);
//            os <<  N ; os << "    " ;
//            os.width(20);
//            os <<  mat(M-1,N-1) << "\n";
//         }

        os.setf(olda,ios::adjustfield);
        os.setf(oldf,ios::floatfield);
        os.precision(oldp);

        return os;
}

/***************************************/
/*  Matrix-MV_Vector multiplication    */
/***************************************/

VECTOR_double CompRow_Mat_double::operator*(const VECTOR_double &x) 
        const
{
        int M = dim_[0];
        int N = dim_[1];

//      Check for compatible dimensions:
        if (x.size() != N) 
        {
           cerr << "Error in CompRow Matvec -- incompatible dimensions." 
                << endl;
           //exit(1);
	   abort();
	   return x;      // return to suppress compiler warning message
         }

         VECTOR_double result(M, 0.0);
         VECTOR_double work(M);
  
         int descra[9];
         descra[0] = 0;
         descra[1] = 0;
         descra[2] = 0;

         F77NAME(dcsrmm) (0, M, 1, N, 1.0,
                  descra, &val_(0), &colind_(0), &rowptr_(0),
                  &x(1), N, 1.0, &result(0), M,
                  &work(0), M);

         return result;
}


/*************************************************/
/* Matrix-Transpose-MV_Vector multiplication...  */
/*************************************************/

VECTOR_double CompRow_Mat_double::trans_mult(const VECTOR_double &x)  
          const
{
          int M = dim_[0];
          int N = dim_[1];

//        Check for compatible dimensions:
          if (x.size() != M) 
          {
             cerr << "Error in CompRow Matvec -- incompatible dimensions." 
                  << endl;
	     abort();
             //exit(1);
             return x;   // return to suppress compiler warning message
           }

           VECTOR_double result(N, 0.0);
           VECTOR_double work(N);
  
           int descra[9];
           descra[0] = 0;
           descra[1] = 0;
           descra[2] = 0;

           F77NAME(dcsrmm) (1, N, 1, M, 1.0,
                        descra, &val_(0), &colind_(0), &rowptr_(0),
                    &x(0), M, 1.0, &result(1), N,
                    &work(1), N);

           return result;
}


CompRow_Mat_double &CompRow_Mat_double::operator*=(double a) {
  for(int i=0; i<nz_; i++)
    val_[i] *= a;		// should use blas
  return *this;
}

CompRow_Mat_double operator*(double a, const CompRow_Mat_double &mat) {
  CompRow_Mat_double result(mat);
  result *= a;
  return result;
}

CompRow_Mat_double operator*(const CompRow_Mat_double &mat, double a) {
  return a*mat;
}



CompRow_Mat_double * CompRow_Mat_double::add(const CompRow_Mat_double &b) const {

  /*
    
    ---For each row_ptr entry (for each row) of *this (which is called a) 
    and b (the argument of the function) determine--
    the first non-zero and last non-zero column entries for *this and b. 
    If they are the same in both, the row is empty and skip to the next row, and set row_ptr of the result. If not:

    &&&--For each row:
    Set a pointer to the first non-zero entry of *this and b.
    Retrieve the smallest value of both pointers.
    ***--If they are both the same, sum associated vals and increase the 
    pointers in both.
    If not, transfer the value of the lowest column, non-zero entry, 
    set the col_ind value of the result and increase the pointers of the 
    lowest column. 
    If the pointer that is being increased has reached the last non-zero
    entry do not increase it.
    If either of both pointers have not reached the last non-zero entries
    go to ***.
    If both pointers have reached their respective last non-zero entries,
    go to &&&.
    
    ---When the last row has been reached, set the number of non-zero entries
    into the last entry of the resultant row_ptr vector.
    
    ---Cast a CompRow_Mat_double with the sum vector objects.

    --return sum
  */
  if ((ncols() != b.ncols()) || (nrows() != b.nrows()))
    throw ErrProgrammingError("Can't sum two matrices of differing sizes!!!",
			      __FILE__, __LINE__);


  std::vector<double> val_result;
  std::vector<int> col_ind_result;
  std::vector<int> row_ptr_result;

  //reserve space for resultant matrix
  //consider worst case scenario: every entry of a and b
  //is mutually "non-overlapping"
  val_result.reserve(NumNonzeros() + b.NumNonzeros());
  col_ind_result.reserve(NumNonzeros() + b.NumNonzeros());
  row_ptr_result.reserve(nrows()+1);//last entry is the number of non-zeros
  int non_zero_entry_number = 0;
  // std::cerr << "A.val=\n" << val_ << std::endl;
  // std::cerr << "A.rowptr=\n" << rowptr_ << std::endl;
  // std::cerr << "A.col=\n" << colind_ << std::endl;

  for (int row_num = 0; row_num< nrows(); row_num++) 
    //loop over all the entries of rowptr_ from this vector
    {

      // std::cerr << "row number =" << row_num << std::endl;
      //global index for entry in val vector
      //a_index and b_index points into the first entry in
      //the col_ind array that is NON-ZERO
      int a_index = row_ptr(row_num);
      int b_index = b.row_ptr(row_num);

      int a_maximum, b_maximum;
      //flags defining the state of the of each row
      bool first_non_zero_col = true; 
      bool a_last_item_reached = false;
      bool b_last_item_reached = false;


      int da = row_ptr(row_num+1) - a_index;
      int db = b.row_ptr(row_num+1) - b_index;
      // std::cerr << "da= " << da << std::endl;
      // std::cerr << "db=" << db << std::endl;
      if (da>0)
	a_maximum = a_index + da -1;
      else
	{
	  //the row is empty
	  a_maximum = a_index;
	  a_last_item_reached = true;
	}
      if (db>0)
	b_maximum = b_index + db -1;
      else
	{
	  //the row is empty
	  b_maximum = b_index;
	  b_last_item_reached = true;
	}

      
      

      // for fixed row, find the smallest column index
      // of a and b. If they are the same, sum the entries
      while(!a_last_item_reached || !b_last_item_reached)
	{
	  int a_column, b_column, a_col_maximum, b_col_maximum;
	  double temp_val = 0;
	  int temp_col = 0;
	  int temp_row = 0;
	  if (da>0)
	    {
	      a_column = col_ind(a_index);
	      a_col_maximum = col_ind(a_maximum);
	    }	  
	  else
	    {
	      a_column = ncols();
	      a_col_maximum = ncols();
	    }
	  if (db>0)
	    {
	      b_column = b.col_ind(b_index);
	      b_col_maximum = b.col_ind(b_maximum);
	    }
	  else
	    {
	      b_column = b.ncols();
	      b_col_maximum = b.ncols();
	    }
	  if(a_column > a_col_maximum)
	    a_last_item_reached = true;
	  if(b_column > b_col_maximum)
	    b_last_item_reached = true;
	  // std::cerr << "******* a_column=" << a_column << std::endl;
	  // std::cerr << "******* b_column=" << b_column << std::endl;
	  
	  //next, check the possible cases
	  if (!a_last_item_reached && !b_last_item_reached)
	    //if neither A and B have reached the end of the row
	    {
	      if(a_column == b_column)
		{
		  // std::cerr << "transfer entries with same index= " << b.val(b_index) + val(a_index) << std::endl;
		  temp_val = b.val(b_index) + val(a_index);
		  temp_col = a_column; //could have used b_column instead
		  temp_row = non_zero_entry_number;
		  if(a_column == a_col_maximum)
		    a_last_item_reached = true;
		  if(b_column == b_col_maximum)
		    b_last_item_reached = true;
		  non_zero_entry_number++;
		  //update pointers and indices
		  if (a_index< a_maximum)
		    a_index++;
		  a_column = col_ind(a_index);
		  if (b_index < b_maximum)
		    b_index++;
		  b_column = b.col_ind(b_index);
		} //end of if(a_column == b_column)
	      else
		{
		  if(a_column < b_column)
		    {
		      // std::cerr << "transfer from A matrix=" << val(a_index) << std::endl;
		      temp_val = val(a_index);
		      temp_col = a_column; 
		      temp_row = non_zero_entry_number;
		      non_zero_entry_number++;
		      //update pointers and indices
		      if(a_column == a_col_maximum)
			a_last_item_reached = true;
		      if (a_index< a_maximum)
			a_index++;
		      a_column = col_ind(a_index);
		    }
		  else
		    {
		      if (b_column< a_column)
			{
			  // std::cerr << "transfer from B matrix=" << b.val(b_index) << std::endl;
			  temp_val = b.val(b_index);
			  temp_col = b_column; 
			  temp_row = non_zero_entry_number; 
			  non_zero_entry_number++;
			  if(b_column == b_col_maximum)
			    b_last_item_reached = true;
			  //update pointers and indices
			  if (b_index< b_maximum)
			    b_index++;
			  b_column = b.col_ind(b_index);
			}
		    }
		}
	      
	    }
	  else
	    {
	      if(a_last_item_reached && !b_last_item_reached)
		//if A has reached the end of the row ONLY
		{
		  // std::cerr << "transfer from B matrix only=" << b.val(b_index) << std::endl;
		  temp_val = b.val(b_index);
		  temp_col = b_column; 
		  temp_row = non_zero_entry_number; 
		  non_zero_entry_number++;
		  if(b_column == b_col_maximum)
		    b_last_item_reached = true;
		  //update pointers and indices
		  if (b_index< b_maximum)
		    b_index++;
		  b_column = b.col_ind(b_index);
		}
	      else
		{
		  if(!a_last_item_reached && b_last_item_reached)
		    //if B has reached the end of the row ONLY
		    {
		      // std::cerr << "transfer from A matrix only=" << val(a_index) << std::endl;
		      temp_val = val(a_index);
		      temp_col = a_column; 
		      temp_row = non_zero_entry_number; 
		      non_zero_entry_number++;
		      if(a_column == a_col_maximum)
			a_last_item_reached = true;
		      //update pointers and indices
		      if (a_index< a_maximum)
			a_index++;
		      a_column = col_ind(b_index);
		    }
		  else
		    {
		      //both have reached the end of the row and should start a new row
		    }
		}
	    }
	  
	  
	  //set non-zero entries in the resultant matrix
	  if(temp_val !=0)
	    //guarantee that the constructed entry is non-zero
	    {
	      // std::cerr << "new entry="<<temp_val << std::endl;
	      // std::cerr << "col_ind=" << temp_col << std::endl;
	      // std::cerr << "row_ptr=" << temp_row << std::endl;
	      val_result.push_back(temp_val);
	      col_ind_result.push_back(temp_col);
	      if (first_non_zero_col)
		{
		  row_ptr_result.push_back(temp_row);
		  first_non_zero_col = false;
		  
		}
	    }
	  
	  
	  
	    
	  // std::cerr << "+++++indices at the end of while loop++++" << std::endl;
	  // std::cerr << "a_index=" << a_index << " a_maximum="<< a_maximum << std::endl;
	  // std::cerr << "b_index=" << b_index << " b_maximum=" << b_maximum << std::endl;
	  // std::cerr << "+++++indices at the end of while loop++++" << std::endl;
	}//end while loop
      if (da==0 && db==0)
	{
	  // std::cerr << "setting empty row" << std::endl;
	  // std::cerr << "row_ptr_result = " << non_zero_entry_number << std::endl;
	  row_ptr_result.push_back(non_zero_entry_number);
	}


      

    }//end of for loop
  //transfer the sum vector components to result
  row_ptr_result.push_back(non_zero_entry_number);//last item in this vector should be non-zero entries
  // std::cerr << "val_result=" << val_result << std::endl;
  // std::cerr << "row_ptr_result=" << row_ptr_result << std::endl;
  // std::cerr << "col_ind_result=" << col_ind_result << std::endl;
  double * valPtr = &val_result[0];
  int * rowPtr = &row_ptr_result[0];
  int * colPtr = &col_ind_result[0];
  CompRow_Mat_double * result = new CompRow_Mat_double(nrows(), ncols(), non_zero_entry_number, valPtr, rowPtr, colPtr);
  return result;
}


// Iterator function definitions.

CompRow_Mat_double_const_iterator::
CompRow_Mat_double_const_iterator(const CompRow_Mat_double &mtx, int row)
  : A(mtx), row_(row) {
  index_ = A.row_ptr(row_);
}

CompRow_Mat_double_const_iterator::
CompRow_Mat_double_const_iterator(const CompRow_Mat_double &mtx) 
  : A(mtx), row_(0) {
  index_ = A.row_ptr(0);
}


bool CompRow_Mat_double_const_iterator::
operator==(const CompRow_Mat_double_const_iterator &o) const {
  return index_==o.index_; 
}


bool CompRow_Mat_double_const_iterator::
operator!=(const CompRow_Mat_double_const_iterator &o) const {
  return index_!=o.index_; 
}

bool CompRow_Mat_double_const_iterator::
operator<(const CompRow_Mat_double_const_iterator &o) const {
  return index_<o.index_; 
} 

void CompRow_Mat_double_const_iterator::operator++() {
  if ( index_ <= A.nnz()) {
    index_++;
    if ( index_ >= A.row_ptr(row_) )
      row_++;
  }
}

double CompRow_Mat_double_const_iterator::operator*() const {
  return A.val(index_); 
}

int CompRow_Mat_double_const_iterator::row() const { 
  return row_; 
}

int CompRow_Mat_double_const_iterator::col() const { 
  return A.col_ind(index_); 
}

void CompRow_Mat_double::clear()
{
  val_.clear();
}
