// -*- C++ -*-
// $RCSfile: forcedensity.C,v $
// $Revision: 1.2 $
// $Author: langer $
// $Date: 2005/06/15 21:14:37 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "common/coord.h"
#include "forcedensity.h"
#include "engine/element.h"
#include "engine/elementnodeiterator.h"
#include "engine/field.h"
#include "engine/flux.h"
#include "engine/indextypes.h"
#include "engine/material.h"
#include "engine/femesh.h"
#include "common/space.h"
#include "common/trace.h"
#include "common/ooferror.h"
using namespace std;

ForceDensity::ForceDensity(PyObject *reg, const string &nm, double x, double y, double z)
  : Property(nm,reg),
    gx(x),
    gy(y),
    gz(z)
{
  displacement = dynamic_cast<TwoVectorField*>(Field::getField("Displacement"));
  stress_flux = dynamic_cast<SymmetricTensorFlux*>(Flux::getFlux("Stress"));
}

void ForceDensity::precompute() {
}

int ForceDensity::K_integration_order(int degree) const {
  return degree;
}

void ForceDensity::fluxrhs(const FEMesh *mesh, const Element *element,
			      const Flux *flux, FluxData *fluxdata,
			      const MasterPosition &x) const 
{
  
  // For mass density, only need to set the appropriate vector,
  // it's the same at every point. 
  
  if(*flux != *stress_flux) {
    throw ErrProgrammingError("Unexpected flux", __FILE__, __LINE__);
  }
  
  Trace("ForceDensity::fluxrhs"); 
  fluxdata->rhs_element(0) -= gx;
  fluxdata->rhs_element(1) -= gy;
  fluxdata->rhs_element(0) -= gz;
}

